from crispy_forms.helper import FormHelper
from crispy_forms.layout import Field, Layout, Submit
from django import forms
from markdownify import markdownify as md

from .models import Entry


class EntryForm(forms.ModelForm):
    class Meta:
        model = Entry
        fields = ("title", "excerpt", "content")

    def __init__(self, *args, **kwargs):
        text = kwargs.pop("button_text", "Submit")
        submit_url = kwargs.pop("submit_url", None)
        super().__init__(*args, **kwargs)
        self.helper = FormHelper(self)
        self.helper.form_action = submit_url
        self.helper.layout = Layout(
            Field("title"),
            Field("excerpt"),
            Field("content"),
            Submit(text, text),
        )

    def clean_content(self):
        """Removed bad tags, e.g. <script>"""
        content = self.cleaned_data["content"]
        cleaned = md(content, strip=["script"])
        return cleaned



from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


def validate_capitalized(value):
    if not value[0].isupper():
        err_msg = _("%(value)s should start with an uppercase letter")
        raise ValidationError(err_msg, params={"value": value})


def validate_and_vs_ampersand(value):
    if "&" in value:
        err_msg = _("%(value)s should use 'and' instead of '&'")
        raise ValidationError(err_msg, params={"value": value})



import bleach
from markdown import Markdown


def convert_md_to_html(raw_markdown_text: str) -> str:
    markdowner = Markdown(extensions=["tables", "footnotes"])
    html = markdowner.convert(raw_markdown_text)
    allowed = [
        "blockquote",
        "h1",
        "h2",
        "h3",
        "b",
        "strong",
        "i",
        "em",
        "p",
        "sup",
        "a",
        "ol",
        "ul",
        "li",
        "table",
        "thead",
        "tbody",
        "th",
        "tr",
        "td",
        "pre",
        "mark",
    ]
    return bleach.clean(html, tags=allowed)



# Generated by Django 3.2.9 on 2021-12-04 08:37

import uuid

import django.db.models.deletion
import django_extensions.db.fields
from django.conf import settings
from django.db import migrations, models

import entries.validators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Entry",
            fields=[
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                (
                    "slug",
                    django_extensions.db.fields.AutoSlugField(
                        blank=True,
                        editable=False,
                        populate_from=["title", "id"],
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Title of entry.",
                        max_length=50,
                        validators=[
                            entries.validators.validate_capitalized,
                            entries.validators.validate_and_vs_ampersand,
                        ],
                    ),
                ),
                (
                    "excerpt",
                    models.CharField(
                        help_text="Short blurb describing entry displayed in list of entries.",
                        max_length=50,
                        validators=[
                            entries.validators.validate_capitalized,
                            entries.validators.validate_and_vs_ampersand,
                        ],
                    ),
                ),
                (
                    "content",
                    models.TextField(
                        help_text="Markdown text formatting, e.g. ##, *, 1., -, etc."
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="entries",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Entry",
                "verbose_name_plural": "Entries",
            },
        ),
    ]



from http import HTTPStatus

import pytest
from django.urls import reverse
from markdownify import markdownify as md

from entries.utils import convert_md_to_html


def test_script_tag_allowed_in_entry(test_entry):
    assert '<script type="text/javascript">' not in test_entry.content


def test_script_tag_removed_by_function(sample_md_bad_tag):
    problem = '<script type="text/javascript">'
    assert problem in sample_md_bad_tag
    assert problem not in md(sample_md_bad_tag, strip=["script"])


def test_script_tag_stripped(test_entry):
    html = convert_md_to_html(test_entry.content)
    assert html == "<h1>This is a header script</h1>\n<p>test</p>"


@pytest.mark.django_db
@pytest.mark.parametrize(
    "named_route, template_name",
    [
        ("entries:view_home", "home.html"),
        ("entries:view_about", "about.html"),
        ("entries:list_entries", "entry_list.html"),
    ],
)
def test_ok_entry_views(client, named_route, template_name):
    url = reverse(named_route)
    response = client.get(url)
    assert HTTPStatus.OK == response.status_code
    assert template_name == response.template_name



import pytest

from entries.forms import EntryForm

START_WITH_UPPER = lambda x: f"{x} should start with an uppercase letter"
NO_AMPERSAND = lambda x: f"{x} should use 'and' instead of '&'"
BAD_TAG = '<script type="text/javascript">'


@pytest.mark.parametrize(
    "field, formatter, value",
    [
        ("title", START_WITH_UPPER, "a lowercase title"),
        ("title", NO_AMPERSAND, "X & Y in title"),
        ("excerpt", START_WITH_UPPER, "a lowercase excerpt"),
        ("excerpt", NO_AMPERSAND, "X & Y in excerpt"),
    ],
)
def test_entry_field_invalid_input_found(field, formatter, value):
    form = EntryForm(data={field: value})
    assert form.errors[field] == [formatter(value)]


def test_title_all_errors():
    x = "a & Y."
    form = EntryForm(data={"title": x})
    assert form.errors["title"] == [START_WITH_UPPER(x), NO_AMPERSAND(x)]


def test_valid_content(sample_data):
    form = EntryForm(data=sample_data)
    assert BAD_TAG in sample_data["content"]
    assert form.is_valid()  # will call validation functions
    assert BAD_TAG not in form.cleaned_data["content"]



from http import HTTPStatus

import pytest
from django.template.response import TemplateResponse
from django.urls import reverse


def test_entry_slug_joining_title_with_id(test_entry, sample_slug):
    assert test_entry.slug.startswith(sample_slug)


@pytest.fixture
def test_detail_entry_endpoint(client, test_entry):
    response = client.get(f"/entry/{test_entry.slug}")
    assert isinstance(response, TemplateResponse)
    assert response.status_code == HTTPStatus.OK
    assert response.template_name == "entry_detail.html"


@pytest.fixture
def test_detail_entry_route(client, test_entry):
    url = reverse("entries:view_entry", kwargs={"slug": test_entry.slug})
    response = client.get(url)
    assert isinstance(response, TemplateResponse)
    assert response.status_code == HTTPStatus.OK
    assert response.template_name == "entry_detail.html"



import pytest
from markdownify import markdownify as md

from entries.models import Entry


@pytest.fixture
def sample_user(django_user_model):
    return django_user_model.objects.create(
        username="userjohn",
        first_name="john",
        last_name="doe",
        password="qwer1234qwer1234",
    )


@pytest.fixture
def modified_first_name():
    return "Juan"


@pytest.fixture
def modified_last_name():
    return "de la Cruz"


@pytest.fixture
def modified_name(modified_first_name, modified_last_name, sample_user):
    """Implies existence of `test_entry` which will be modified"""
    return {
        "first_name": modified_first_name,
        "last_name": modified_last_name,
    }


@pytest.fixture
def other_user(django_user_model):
    return django_user_model.objects.create(
        username="userjames",
        first_name="james",
        last_name="delacruz",
        password="qwer1234qwer1234",
    )


@pytest.fixture
def sample_title():
    return "Sample valid title"


@pytest.fixture
def sample_slug():
    return "sample-valid-title"


@pytest.fixture
def sample_excerpt():
    return "Sample description."


@pytest.fixture
def sample_md_bad_tag():
    return """# This is a header script\r\n\r\n<script type="text/javascript">test</script>"""


@pytest.fixture
def sample_data(sample_title, sample_excerpt, sample_md_bad_tag):
    return {
        "title": sample_title,
        "excerpt": sample_excerpt,
        "content": sample_md_bad_tag,
    }


@pytest.fixture
def test_entry(sample_user, sample_data):
    return Entry.objects.create(
        title=sample_data["title"],
        excerpt=sample_data["excerpt"],
        content=md(sample_data["content"], strip=["script"]),
        author=sample_user,
    )


@pytest.fixture
def modified_data(test_entry, sample_excerpt, sample_md_bad_tag):
    """Implies existence of `test_entry` which will be modified"""
    return {
        "title": "Modified valid title",
        "excerpt": sample_excerpt,
        "content": sample_md_bad_tag,
    }



from http import HTTPStatus

import pytest
from django.core.exceptions import PermissionDenied
from django.http.response import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import reverse

ENDPOINT = lambda x: f"/entry/edit/{x}"
ROUTE = lambda x: reverse("entries:edit_entry", kwargs={"slug": x})


@pytest.mark.parametrize("formatter", [ENDPOINT, ROUTE])
def test_edit_entry_anonymous_redirected(client, sample_slug, formatter):
    url = formatter(sample_slug)
    response = client.get(url)
    assert isinstance(response, HttpResponseRedirect)
    assert HTTPStatus.FOUND == response.status_code


@pytest.mark.parametrize("formatter", [ENDPOINT, ROUTE])
def test_edit_entry_forbidden_even_if_authenticated(
    client, other_user, test_entry, formatter
):
    client.force_login(other_user)
    url = formatter(test_entry.slug)
    client.get(url)
    with pytest.raises(PermissionDenied):
        raise PermissionDenied


@pytest.mark.parametrize("formatter", [ENDPOINT, ROUTE])
def test_edit_entry_get_authenticated(
    client, sample_user, test_entry, formatter
):
    client.force_login(sample_user)
    url = formatter(test_entry.slug)
    response = client.get(url)
    assert isinstance(response, TemplateResponse)
    assert HTTPStatus.OK == response.status_code
    assert "entry_edit.html" == response.template_name


@pytest.mark.parametrize("formatter", [ENDPOINT, ROUTE])
def test_edit_entry_post_authenticated(
    client, modified_data, sample_user, test_entry, formatter
):
    url = formatter(test_entry.slug)
    client.force_login(sample_user)
    response = client.post(url, data=modified_data)
    assert isinstance(response, HttpResponseRedirect)
    assert HTTPStatus.FOUND == response.status_code
    assert response.url.startswith(f"/entry/view/{test_entry.slug}")



from django.urls import path

from .views import (
    add_entry,
    delete_entry,
    edit_entry,
    list_entries,
    view_about,
    view_entry,
    view_home,
)

app_name = "entries"
urlpatterns = [
    path("entries/list", list_entries, name="list_entries"),
    path("create/add", add_entry, name="add_entry"),
    path("edit/<slug:slug>", edit_entry, name="edit_entry"),
    path("delete/<slug:slug>", delete_entry, name="delete_entry"),
    path("view/<slug:slug>", view_entry, name="view_entry"),
    path("about/", view_about, name="view_about"),
    path("", view_home, name="view_home"),
]



from http import HTTPStatus

import pytest
from django.core.exceptions import PermissionDenied
from django.http.response import HttpResponseRedirect
from django.urls import reverse

ENDPOINT = lambda x: f"/entry/delete/{x}"
ROUTE = lambda x: reverse("entries:delete_entry", kwargs={"slug": x})
LIST_URL = reverse("entries:list_entries")


@pytest.mark.parametrize("formatter", [ENDPOINT, ROUTE])
def test_del_entry_anonymous_redirected(client, sample_slug, formatter):
    url = formatter(sample_slug)
    response = client.delete(url)
    assert isinstance(response, HttpResponseRedirect)
    assert HTTPStatus.FOUND == response.status_code


@pytest.mark.parametrize("formatter", [ENDPOINT, ROUTE])
def test_del_entry_forbidden_even_if_authenticated(
    client, other_user, test_entry, formatter
):
    url = formatter(test_entry.slug)
    client.force_login(other_user)
    client.delete(url)
    with pytest.raises(PermissionDenied):
        raise PermissionDenied


@pytest.mark.parametrize("formatter", [ENDPOINT, ROUTE])
def test_del_entry_authenticated(client, sample_user, test_entry, formatter):
    url = formatter(test_entry.slug)
    client.force_login(sample_user)
    response = client.delete(url)
    assert response.status_code == HTTPStatus.OK
    assert response.headers["HX-Redirect"] == LIST_URL



from http import HTTPStatus

import pytest
from django.http.response import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import reverse

ENDPOINT = "/entry/create/add"
ROUTE = reverse("entries:add_entry")


@pytest.mark.parametrize("url", [ENDPOINT, ROUTE])
def test_add_entry_anonymous_redirected(client, url):
    response = client.get(url)
    assert isinstance(response, HttpResponseRedirect)
    assert HTTPStatus.FOUND == response.status_code


@pytest.mark.parametrize("url", [ENDPOINT, ROUTE])
def test_add_entry_get_authenticated(client, sample_user, url):
    client.force_login(sample_user)
    response = client.get(url)
    assert isinstance(response, TemplateResponse)
    assert HTTPStatus.OK == response.status_code
    assert "entry_edit.html" == response.template_name


@pytest.mark.django_db
@pytest.mark.parametrize("url", [ENDPOINT, ROUTE])
def test_add_entry_post_authenticated(
    client, sample_data, sample_user, sample_slug, url
):
    client.force_login(sample_user)
    response = client.post(url, data=sample_data)
    assert isinstance(response, HttpResponseRedirect)
    assert HTTPStatus.FOUND == response.status_code
    assert response.url.startswith(f"/entry/view/{sample_slug}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['entries', 'entries.migrations', 'entries.tests']

package_data = \
{'': ['*'], 'entries': ['static/css/*', 'templates/*']}

install_requires = \
['Django>=3.2,<4.0',
 'Markdown>=3.3.6,<4.0.0',
 'bleach>=4.1.0,<5.0.0',
 'django-crispy-forms>=1.13.0,<2.0.0',
 'django-extensions==3.1.5',
 'markdownify>=0.10.0,<0.11.0',
 'types-Markdown>=3.3.8,<4.0.0',
 'types-bleach>=4.1.1,<5.0.0']

setup_kwargs = {
    'name': 'django-entries',
    'version': '0.1.5',
    'description': 'Entries is a helper Django app with CRUD functions based on htmx.',
    'long_description': "# Entries\n\n## Overview\n\nEntries is a Django app that has basic create-read-update-delete functionality for a model that `Entry` consisting of the following fields: `title`, `excerpt`,`content` and `author`.\n\nThe [templates/base.html](./entries/templates/base.html) provides a light set of tools:\n\n1. `starter.css` [stylesheet](./entries/static/css/starter.css) for some defaults\n2. `pylon` 0.1.1 for `<hstack>` and `<vstack>` layouts\n3. `htmx` 1.6.1 for html-over-the-wire functionality\n4. `hyperscript` 0.9 for client-side reactivity\n5. `simplemde` a simple text editor that accepts markdown\n\n## Quickstart\n\n```python\n# in project_folder/settings.py\nINSTALLED_APPS = [\n    ...,\n    'crispy_forms',  # crispy_forms at least > v1.13\n    'entries' # new\n]\n\n# in project_folder/urls.py\nfrom django.urls import path, include # new\nurlpatterns = [\n    ...,\n    path('entry/', include('entries.urls')) # new\n]\n```\n\nAdd to database:\n\n```zsh\n.venv> python manage.py migrate # enter virtual environment\n```\n",
    'author': 'Marcelino G. Veloso III',
    'author_email': 'mars@veloso.one',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from django.contrib.auth.decorators import login_required
from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse
from django.urls import reverse
from django.views.decorators.http import require_http_methods

from .forms import EntryForm
from .models import Entry


@login_required
def edit_entry(request: HttpRequest, slug: str) -> TemplateResponse:
    entry = Entry.get_for_user(slug, request.user)
    url = reverse("entries:edit_entry", kwargs={"slug": slug})
    form = EntryForm(request.POST or None, instance=entry, submit_url=url)
    if request.method == "POST" and form.is_valid():
        if form.has_changed():
            form.save()
        return redirect(entry.get_absolute_url())
    return TemplateResponse(
        request,
        "entry_edit.html",
        {
            "form": form,
            "edit_header": f"Update {entry.title}",
        },
    )


@login_required
def add_entry(request: HttpRequest) -> TemplateResponse:
    url = reverse("entries:add_entry")
    form = EntryForm(request.POST or None, submit_url=url)
    if request.method == "POST" and form.is_valid():
        entry = form.save(commit=False)
        entry.author = request.user
        entry.save()
        return redirect(entry.get_absolute_url())
    return TemplateResponse(
        request,
        "entry_edit.html",
        {
            "form": form,
            "edit_header": "Write An Entry",
        },
    )


@login_required
@require_http_methods(["DELETE"])
def delete_entry(request: HttpRequest, slug: str) -> HttpResponse:
    url = reverse("entries:list_entries")
    entry = Entry.get_for_user(slug, request.user)
    response = HttpResponse(headers={"HX-Redirect": url})
    entry.delete()
    return response


def view_entry(request: HttpRequest, slug: str) -> TemplateResponse:
    return TemplateResponse(
        request,
        "entry_detail.html",
        {"entry": get_object_or_404(Entry, slug=slug)},
    )


def list_entries(request: HttpRequest) -> TemplateResponse:
    return TemplateResponse(
        request, "entry_list.html", {"entries": Entry.objects.all()}
    )


def view_about(request: HttpRequest) -> TemplateResponse:
    return TemplateResponse(request, "about.html", {})


def view_home(request: HttpRequest) -> TemplateResponse:
    return TemplateResponse(request, "home.html", {})



import uuid

from django.contrib.auth import get_user_model
from django.core.exceptions import PermissionDenied
from django.db import models
from django.shortcuts import get_object_or_404
from django.template import Context, Template
from django.urls import reverse
from django.utils.safestring import mark_safe
from django_extensions.db.fields import AutoSlugField
from django_extensions.db.models import TimeStampedModel

from .utils import convert_md_to_html
from .validators import validate_and_vs_ampersand, validate_capitalized


class Entry(TimeStampedModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4)
    slug = AutoSlugField(populate_from=["title", "id"])
    title = models.CharField(
        max_length=50,
        help_text="Title of entry.",
        validators=[validate_capitalized, validate_and_vs_ampersand],
    )
    excerpt = models.CharField(
        max_length=50,
        help_text="Short blurb describing entry displayed in list of entries.",
        validators=[validate_capitalized, validate_and_vs_ampersand],
    )
    content = models.TextField(
        help_text="Markdown text formatting, e.g. ##, *, 1., -, etc."
    )
    author = models.ForeignKey(
        get_user_model(), on_delete=models.CASCADE, related_name="entries"
    )

    class Meta:
        verbose_name = "Entry"
        verbose_name_plural = "Entries"

    def __str__(self):
        return f"{self.title} by {self.author}"

    def get_absolute_url(self):
        return reverse("entries:view_entry", kwargs={"slug": self.slug})

    @property
    def meta(self):
        data = {
            "title": self.title,
            "description": self.excerpt,
            "url": self.get_absolute_url(),
        }
        html = """
            <title>{{title}}</title>
            <meta property="og:title" content="{{title}}">
            <meta name="description" property="og:description" content="{{description|striptags|truncatewords:20}}">
            <meta property="og:url" content="{{url}}">
            <meta property="og:type" content="blog">
        """
        context = Context(data)
        template = Template(html)
        return mark_safe(template.render(context))

    @property
    def md_content(self) -> str:
        return mark_safe(convert_md_to_html(self.content))

    @classmethod
    def get_for_user(cls, slug: str, user):
        """Simple permission checking"""
        entry = get_object_or_404(cls, slug=slug)
        if entry.author != user:
            raise PermissionDenied()
        return entry
